// SPDX-License-Identifier: LGPL-3.0-or-later
// See Notices.txt for copyright information

pub mod array_vec;

pub(crate) const fn range_u32_len(range: &std::ops::Range<u32>) -> usize {
    let retval = range.end.saturating_sub(range.start);
    assert!(retval as usize as u32 == retval, "len overflowed");
    retval as usize
}

pub(crate) const fn range_u32_nth(range: &std::ops::Range<u32>, index: usize) -> Option<u32> {
    if index < range_u32_len(range) {
        Some(range.start + index as u32)
    } else {
        None
    }
}

pub(crate) const fn range_u32_nth_or_panic(range: &std::ops::Range<u32>, index: usize) -> u32 {
    if let Some(retval) = range_u32_nth(range, index) {
        retval
    } else {
        panic!("index out of range")
    }
}

pub(crate) const fn range_intersection(
    a: &std::ops::Range<u32>,
    b: &std::ops::Range<u32>,
) -> Option<std::ops::Range<u32>> {
    let start = if a.start > b.start { a.start } else { b.start };
    let end = if a.end < b.end { a.end } else { b.end };
    if start < end {
        Some(start..end)
    } else {
        None
    }
}
